from enum import Enum


class CheckResult(str, Enum):
    PASSED = "PASSED"
    FAILED = "FAILED"
    # Unknown should be used when a check does not wish to return a result, generally due to the inability
    # to resolve a value or similar types of errors.
    UNKNOWN = "UNKNOWN"
    # Skipped is used by the framework when a test is suppressed and should not be used directly by checks.
    SKIPPED = "SKIPPED"


class CheckCategories(Enum):
    LOGGING = 1
    ENCRYPTION = 2
    GENERAL_SECURITY = 3
    NETWORKING = 4
    IAM = 5
    BACKUP_AND_RECOVERY = 6
    CONVENTION = 7
    SECRETS = 8
    KUBERNETES = 9
    APPLICATION_SECURITY = 10
    SUPPLY_CHAIN = 11


class OutputFormat(Enum):
    CONSOLE = 1
    JSON = 2
    JUNIT_XML = 3


class ContextCategories(Enum):
    PROVIDER = 1
    TERRAFORM = 2
    LOCALS = 3
    MODULE = 4
    DATA = 5
    RESOURCE = 6
    VARIABLE = 7
    OUTPUT = 8



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class TimestreamDatabaseKMSKey(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Timestream database is encrypted with KMS CMK"
        id = "CKV_AWS_160"
        supported_resources = ["aws_timestreamwrite_database"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "kms_key_id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = TimestreamDatabaseKMSKey()



from checkov.common.checks_infra.solvers.attribute_solvers import *
from checkov.common.checks_infra.solvers.complex_solvers import *
from checkov.common.checks_infra.solvers.connections_solvers import *
from checkov.common.checks_infra.solvers.filter_solvers import *


import logging
import subprocess  # nosec
from typing import Union, Dict, Any

import docker
import json
import os
import time

from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import docker_image_scanning_integration

TWISTCLI_FILE_NAME = 'twistcli'
DOCKER_IMAGE_SCAN_RESULT_FILE_NAME = 'docker-image-scan-results.json'


def _get_docker_image_name(docker_image_id: str) -> str:
    try:
        docker_client = docker.from_env()
        return docker_client.images.get(docker_image_id).attrs['RepoDigests'][0].split('@')[0]
    except Exception as e:
        logging.error("docker image needs to have repository")
        raise e


def _get_dockerfile_content(dockerfile_path: Union[str, "os.PathLike[str]"]) -> str:
    try:
        with open(dockerfile_path) as f:
            return f.read()
    except FileNotFoundError as e:
        logging.error(f"Path to Dockerfile is invalid\n{e}")
        raise e
    except Exception as e:
        logging.error(f"Failed to read Dockerfile content\n{e}")
        raise e


class ImageScanner:
    def __init__(self) -> None:
        self.docker_image_name = ''
        self.dockerfile_content = ''

    def setup_scan(
        self,
        docker_image_id: str,
        dockerfile_path: Union[str, "os.PathLike[str]"],
        skip_extract_image_name: bool,
    ) -> None:
        try:
            if skip_extract_image_name:
                # Provide a default image name in case the image has not been tagged with a name
                self.docker_image_name = f'repository/image{str(time.time() * 1000)}'
            else:
                self.docker_image_name = _get_docker_image_name(docker_image_id)
            self.dockerfile_content = _get_dockerfile_content(dockerfile_path)

            if not os.path.exists(TWISTCLI_FILE_NAME):
                docker_image_scanning_integration.download_twistcli(TWISTCLI_FILE_NAME)
        except Exception as e:
            logging.error(f"Failed to setup docker image scanning\n{e}")
            raise e

    @staticmethod
    def cleanup_scan() -> None:
        os.remove(TWISTCLI_FILE_NAME)
        logging.info('twistcli file removed')

    @staticmethod
    def run_image_scan(docker_image_id: str) -> Dict[str, Any]:
        command = f"./{TWISTCLI_FILE_NAME} images scan --address {docker_image_scanning_integration.get_proxy_address()} --token {docker_image_scanning_integration.get_bc_api_key()} --details --output-file \"{DOCKER_IMAGE_SCAN_RESULT_FILE_NAME}\" {docker_image_id}"
        logging.debug(f"TwistCLI: {command}")
        subprocess.run(command, check=True, shell=True)  # nosec
        logging.info(f'TwistCLI ran successfully on image {docker_image_id}')

        with open(DOCKER_IMAGE_SCAN_RESULT_FILE_NAME) as docker_image_scan_result_file:
            scan_result = json.load(docker_image_scan_result_file)
        return scan_result

    def scan(self, docker_image_id: str, dockerfile_path: str, skip_extract_image_name: bool = False) -> None:
        try:
            self.setup_scan(docker_image_id, dockerfile_path, skip_extract_image_name)
            scan_result = self.run_image_scan(docker_image_id)
            docker_image_scanning_integration.report_results(
                twistcli_scan_result=scan_result,
                file_path=dockerfile_path,
                file_content=self.dockerfile_content,
                docker_image_name=self.docker_image_name,
            )
            logging.info('Docker image scanning results reported to the platform')
            self.cleanup_scan()
        except Exception as e:
            logging.error(f"Failed to scan docker image\n{e}")
            raise e


image_scanner = ImageScanner()



import os
from abc import ABC, abstractmethod

from checkov.common.bridgecrew.integration_features.integration_feature_registry import integration_feature_registry
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration


class BaseIntegrationFeature(ABC):
    def __init__(self, bc_integration: BcPlatformIntegration, order):
        self.bc_integration = bc_integration
        self.order = order
        integration_feature_registry.register(self)
        self.integration_feature_failures = False

    @abstractmethod
    def is_valid(self):
        raise NotImplementedError()

    def pre_scan(self):
        """Runs before any runners"""
        pass

    def pre_runner(self):
        """Runs before each runner"""
        pass

    def post_runner(self, scan_reports):
        """Runs after each runner completes"""
        pass




from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class TransferServerIsPublic(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Transfer Server is not exposed publicly."
        id = "CKV_AWS_164"
        supported_resources = ['AWS::Transfer::Server']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/EndpointType'

    def get_expected_values(self):
        return ["VPC", "VPC_ENDPOINT"]


check = TransferServerIsPublic()



class BridgecrewAuthError(Exception):
    def __init__(self, message="Authorization error accessing Bridgecrew.cloud api. Please check bc-api-key"):
        self.message = message

    def __str__(self):
        return 'BCAuthError, {0} '.format(self.message)



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck

class AmazonMQBrokerPublicAccess(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure Amazon MQ Broker should not have public access"
        id = "CKV_AWS_69"
        supported_resources = ['AWS::AmazonMQ::Broker']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources, missing_block_result=CheckResult.FAILED)

    def get_expected_value(self):
        return False

    def get_inspected_key(self):
        """
            validates Amazon MQ Broker should not have public access
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-amazonmq-broker.html
        :return: <CheckResult>
        """
        return 'Properties/PubliclyAccessible'


check = AmazonMQBrokerPublicAccess()



import asyncio
import json
import logging
import sys
from collections.abc import Iterable
from pathlib import Path
from typing import List, Tuple

from aiomultiprocess import Pool

from checkov.common.bridgecrew.platform_integration import bc_integration, BcPlatformIntegration
from checkov.common.bridgecrew.vulnerability_scanning.integrations.package_scanning import package_scanning_integration

SUPPORTED_PACKAGE_FILES = {
    "bower.json",
    "build.gradle",
    "build.gradle.kts",
    "go.sum",
    "gradle.properties",
    "METADATA",
    "npm-shrinkwrap.json",
    "package.json",
    "package-lock.json",
    "pom.xml",
    "requirements.txt",
}
TWISTCLI_FILE_NAME = 'twistcli'


class PackageScanner:
    def __init__(self) -> None:
        self.twistcli_path = Path(TWISTCLI_FILE_NAME)

    def setup_scan(self) -> None:
        try:
            if not self.twistcli_path.exists():
                package_scanning_integration.download_twistcli(self.twistcli_path)
        except Exception as e:
            logging.error(f"Failed to setup package scanning\n{e}")
            raise e

    def cleanup_scan(self) -> None:
        if self.twistcli_path.exists():
            self.twistcli_path.unlink()
        logging.info('twistcli file removed')

    async def run_scan(
        self,
        command: str,
        bc_platform_integration: BcPlatformIntegration,
        bc_api_key: str,
        input_path: Path,
        output_path: Path,
    ) -> int:
        logging.info(f"Start to scan package file {input_path}")
        process = await asyncio.create_subprocess_shell(command)

        exit_code = await process.wait()

        if exit_code:
            logging.error(f"Failed to scan package file {input_path}")
            return exit_code

        logging.info(f"Successfully scanned package file {input_path}")

        # read and delete the report file
        scan_result = json.loads(output_path.read_text())
        output_path.unlink()

        exit_code = await package_scanning_integration.report_results_async(
            twistcli_scan_result=scan_result,
            bc_platform_integration=bc_platform_integration,
            bc_api_key=bc_api_key,
            file_path=input_path,
        )

        return exit_code

    async def run(
        self,
        bc_platform_integration: BcPlatformIntegration,
        address: str,
        bc_api_key: str,
        input_output_paths: "Iterable[Tuple[Path, Path]]",
    ) -> List[int]:
        args = [
            (
                f"./{TWISTCLI_FILE_NAME} coderepo scan --address {address} --token {bc_api_key} --output-file \"{output_path.absolute()}\" \"{input_path.absolute()}\"",
                bc_platform_integration,
                bc_api_key,
                input_path,
                output_path,
            )
            for input_path, output_path in input_output_paths
        ]
        async with Pool() as pool:
            exit_codes = await pool.starmap(self.run_scan, args)

        return exit_codes

    def scan(self, code_repo_path: Path) -> None:
        try:
            # backwards compatibility, should be removed in a later stage
            code_repo_path = code_repo_path if isinstance(code_repo_path, Path) else Path(code_repo_path)

            self.setup_scan()

            if code_repo_path.is_file():
                input_output_paths = [(code_repo_path, code_repo_path.parent / f"{code_repo_path.stem}_result.json")]
            else:
                logging.info(f"Searching for package files in {code_repo_path}")

                input_output_paths = [
                    (file_path, file_path.parent / f"{file_path.stem}_result.json")
                    for file_path in code_repo_path.glob("**/*")
                    if file_path.name in SUPPORTED_PACKAGE_FILES
                ]

                logging.info(f"Found {len(input_output_paths)} package files")

            # can not be pickled
            bc_integration.s3_client = None
            bc_integration.http = None

            exit_codes = asyncio.run(
                self.run(
                    bc_platform_integration=bc_integration,
                    address=package_scanning_integration.get_proxy_address(),
                    bc_api_key=package_scanning_integration.get_bc_api_key(),
                    input_output_paths=input_output_paths,
                )
            )

            if any(code != 0 for code in exit_codes):
                sys.exit(1)

            logging.info('Package file scanning results reported to the platform')
            self.cleanup_scan()
        except Exception as e:
            logging.error(f"Failed to scan package files\n{e}")
            raise e



from checkov.terraform.checks.resource.openstack.AbsSecurityGroupUnrestrictedIngress import AbsSecurityGroupUnrestrictedIngress


class SecurityGroupUnrestrictedIngress3389(AbsSecurityGroupUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_OPENSTACK_3", port=3389)


check = SecurityGroupUnrestrictedIngress3389()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class StorageAccountsTransportEncryption(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that 'Secure transfer required' is set to 'Enabled'"
        id = "CKV_AZURE_3"
        supported_resources = ['azurerm_storage_account']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'enable_https_traffic_only'


check = StorageAccountsTransportEncryption()



from os.path import dirname, basename, isfile, join
import glob

modules = glob.glob(join(dirname(__file__), "*.py"))
__all__ = [basename(f)[:-3] for f in modules if isfile(f) and not f.endswith("__init__.py")]



from typing import Optional, TYPE_CHECKING

from typing_extensions import TypedDict

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckResult


class _CheckResult(TypedDict, total=False):
    result: "CheckResult"
    suppress_comment: str


class _SkippedCheck(TypedDict, total=False):
    bc_id: Optional[str]
    id: str
    suppress_comment: str



from typing import Any, List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppSyncFieldLevelLogs(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure AppSync has Field-Level logs enabled"
        id = "CKV_AWS_194"
        supported_resources = ("aws_appsync_graphql_api",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "log_config/[0]/field_log_level"

    def get_expected_values(self) -> List[Any]:
        return ["ALL", "ERROR"]


check = AppSyncFieldLevelLogs()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class EC2EBSOptimized(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that EC2 is EBS optimized"
        id = "CKV_AWS_135"
        supported_resources = ['aws_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "ebs_optimized"


check = EC2EBSOptimized()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServicePythonVersion(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that 'Python version' is the latest, if used to run the web app"
        id = "CKV_AZURE_82"
        supported_resources = ['azurerm_app_service']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "site_config/[0]/python_version/[0]"

    def get_expected_value(self):
        return '3.4'


check = AppServicePythonVersion()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from typing import List


class AzureDefenderOnServers(BaseResourceCheck):
    def __init__(self):
        name = "Ensure that Azure Defender is set to On for Servers"
        id = "CKV_AZURE_55"
        supported_resources = ['azurerm_security_center_subscription_pricing']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        return CheckResult.PASSED if conf.get('resource_type', [None])[0] != 'VirtualMachines' \
                                     or conf.get('tier', [None])[0] == 'Standard' else CheckResult.FAILED

    def get_evaluated_keys(self) -> List[str]:
        return ['resource_type', 'tier']


check = AzureDefenderOnServers()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class LaunchConfigurationEBSEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the Launch configuration EBS is securely encrypted"
        id = "CKV_AWS_8"
        supported_resources = ['AWS::AutoScaling::LaunchConfiguration']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
        Looks for encryption configuration of device block mapping in an AWS launch configurations
        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-as-launchconfig-blockdev-template.html
        :param conf: aws_launch_configuration configuration
        :return: <CheckResult>
        """
        ebs_encryption_confs = []
        if 'Properties' in conf.keys() and conf['Properties'].get('BlockDeviceMappings'):
            for block_device_mapping in conf['Properties']['BlockDeviceMappings']:
                if not isinstance(block_device_mapping, dict):
                    return CheckResult.UNKNOWN
                if block_device_mapping.get('Ebs') and not block_device_mapping.get('VirtualName'):
                    ebs_encryption_confs.append(block_device_mapping['Ebs'].get('Encrypted'))
                else:
                    return CheckResult.FAILED
            if all(ebs_encryption_confs):
                return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.FAILED


check = LaunchConfigurationEBSEncryption()
